#ifndef CODECCONFIG_H
#define CODECCONFIG_H

#include "okdialog.h"
#include <videoencoder.h>

#define HAVE_NEWQT

// ok - MOC can't work with macros
// thus using QWidget for QDial objects
class QLCDNumber;
class QListBox;
class QListView;
class QListViewItem;
class QLineEdit;
class QComboBox;
class QCheckBox;

class QavmCodecDialog : public QavmOkDialog
{
    Q_OBJECT
	;

    class QInputInteger: public QavmOkDialog
    {
	QLineEdit* m_pEdit;
	int _defval;
    public:
	QInputInteger(QWidget* parent, QString title, int defval);
	virtual void accept();
	int value() { return _defval; }
    };

    class QInputString: public QavmOkDialog
    {
	QLineEdit* m_pEdit;
	avm::string m_defval;
    public:
	QInputString(QWidget* parent, QString title, const char* defval);
	virtual void accept();
	const char* value() { return m_defval.c_str(); }
    };

    class QInputSelect: public QavmOkDialog
    {
	QComboBox* m_pBox;
	const avm::vector<avm::string>& _options;
	int _defval;
    public:
	QInputSelect(QWidget* parent, QString title, const avm::vector<avm::string>& options, int defval);
	virtual void accept();
	int value() { return _defval; }
    };


public:
    QavmCodecDialog( QWidget*, const avm::vector<avm::CodecInfo>& codecs,
		    avm::CodecInfo::Direction dir = avm::CodecInfo::Encode );
    ~QavmCodecDialog();
    avm::VideoEncoderInfo getInfo();

    int getCurrent() const;
    void setCurrent(int i);
    void setQuality(int i);
    void setKeyFrame(int i);
public slots:
    virtual void about();
    virtual void accept();
    virtual void changeAttr( QListViewItem* item );
    virtual void clickedAttr( QListViewItem* item );
    virtual void selectCodec();
    virtual void shortNames(int);

    void codecUp() { codecMove( -1 ); }
    void codecDown() { codecMove( 1 ); }
    void codecTop() { codecMove( 0 ); }
    void codecBottom() { codecMove( -1000 ); }

    // Qt2.0 compatibility
    void rightClickedAttr( QListViewItem* item, const QPoint&, int );

protected:
    void addAttributes( const avm::CodecInfo&, const avm::vector<avm::AttributeInfo>& ci );
    void codecMove( int dir );
    void codecUpdateList();
    void createGui();
    void createLCD();
    void createMoveGroup( QWidget* parent );

    const avm::vector<avm::CodecInfo>& m_Codecs;
    avm::vector<unsigned> m_Order;
    avm::CodecInfo::Direction m_Direction;
    QLCDNumber* m_pLCDNumber1;
    QLCDNumber* m_pLCDNumber2;
    QWidget* m_pKeyframe;
    QWidget* m_pQuality;
    QCheckBox* m_pShortBox;
    QListBox* m_pListCodecs;
    QListView* m_pTabAttr;
};

#endif // CODECCONFIG_H
