#include "fillplugins.h"
#ifdef HAVE_LIBDIVXENCORE
#include <encore2.h>
#endif
#include "videodecoder.h"
#include "videoencoder.h"
#include "avm_fourcc.h"
#include "configfile.h"
#include "avm_output.h"
#include "plugin.h"

#include <stdio.h>

AVM_BEGIN_NAMESPACE;

PLUGIN_TEMP(divx4);

static const int DIVX4_MAX_QUALITY = 6;
static const char* strDebugEnc = "DivX4 plugin";

extern "C" int quiet_encore;

#ifdef HAVE_LIBDIVXDECORE
class DIVX_VideoDecoder: public IVideoDecoder, public IRtConfig
{
    unsigned long m_ulHandle;
    int m_iLastPPMode;
    int m_iMaxAuto;
    int m_iLastBrightness;
    int m_iLastContrast;
    int m_iLastSaturation;
//    bool m_bCompatWMV2;
    bool m_bCompat311;
    bool m_bSetFlg;
    char m_cFormatBuf[128];
    avm::vector<AttributeInfo> m_YvAttrs;

public:
    DIVX_VideoDecoder(const CodecInfo& info, const BITMAPINFOHEADER& bh,
		      int flip) :
    IVideoDecoder(info, bh), m_ulHandle(0), m_iLastPPMode(0),
	m_iLastBrightness(0), m_iLastContrast(0), m_iLastSaturation(0),
	m_bSetFlg(true)
    {
	switch(bh.biCompression)
	{
#ifdef USE_311_DECODER
	case fccdiv3:
	case fccDIV3:
	case fccdiv4:
	case fccDIV4:
	case fccdiv5:
	case fccDIV5:
	case fccdiv6:
	case fccDIV6:
	case fccMP41:
	case fccMP43:
	    m_bCompat311=true;
//	    m_bCompatWMV2=false;
	    break;
//	case fccWMV2:
//	    m_bCompat311=true;
//	    m_bCompatWMV2=true;
//	    break;
#endif
	default:
	    m_bCompat311=false;
	    break;
	}
	m_Dest = bh;
	memset(m_cFormatBuf, 0, sizeof(m_cFormatBuf));
	memcpy(m_cFormatBuf,  ((const char*)&bh) + sizeof(BITMAPINFOHEADER),
	       bh.biSize - sizeof(BITMAPINFOHEADER));
	m_Dest.SetBits(24);
	if (flip)
	    m_Dest.biHeight *= -1;

	PluginGetAttrInt(m_Info, divx4str_postprocessing, &m_iLastPPMode);
	PluginGetAttrInt(m_Info, divx4str_maxauto, &m_iMaxAuto);
	PluginGetAttrInt(m_Info, divx4str_brightness, &m_iLastBrightness);
	PluginGetAttrInt(m_Info, divx4str_contrast, &m_iLastContrast);
	PluginGetAttrInt(m_Info, divx4str_saturation, &m_iLastSaturation);

	m_YvAttrs.push_back(AttributeInfo(divx4str_postprocessing,
					  "Image postprocessing mode ( 6 slowest )",
					  AttributeInfo::Integer, 0, 6));
	m_YvAttrs.push_back(AttributeInfo(divx4str_maxauto,
					  "Maximum autoquality level",
					  AttributeInfo::Integer, 0, 6, 6));
    }
    virtual ~DIVX_VideoDecoder()
    {
	Stop();
    }
    virtual CAPS GetCapabilities() const
    {
	return CAPS(CAP_YV12|CAP_YUY2);
    }
    virtual IRtConfig* GetRtConfig()
    {
	return this;
    }
    virtual int Start()
    {
	DEC_PARAM param;

        memset(&param, 0, sizeof(param));
	int csp = m_Dest.biCompression;
	// AVM_WRITE("DivX4 plugin", "csp: 0x%x  %.4s  bitcount: %d height: %d\n", csp, (char*)&csp, m_obh.biBitCount, m_obh.biHeight);

	if (csp == 0 || csp == 3)
	{
	    bool up = (m_Dest.biHeight > 0);
	    switch (m_Dest.biBitCount)
	    {
	    case 16:
		if (csp == 3)
		{
		    param.output_format = (up) ? DEC_RGB565 : DEC_RGB565_INV;
		    break;
		}
                // 16 bit with csp == 0  is 15bit
                /* fall through */
	    case 15:
		param.output_format = (up) ? DEC_RGB555 : DEC_RGB555_INV;
		break;
	    case 24:
		param.output_format = (up) ? DEC_RGB24 : DEC_RGB24_INV;
		break;
	    case 32:
		param.output_format = (up) ? DEC_RGB32 : DEC_RGB32_INV;
		break;
	    default:
		return -1;
	    }
	}
	else
	{
	    switch(csp)
	    {
	    case IMG_FMT_I420:
		param.output_format = DEC_420;
		break;
	    case IMG_FMT_YV12:
		param.output_format = DEC_YV12;
		break;
	    case IMG_FMT_YUY2:
		param.output_format = DEC_YUV2;
		break;
	    case IMG_FMT_I422:
	    case IMG_FMT_UYVY:
		param.output_format = DEC_UYVY;
		break;
	    default:
                return -1;
	    }
	}

	param.x_dim = m_Dest.biWidth;
	param.y_dim = labs(m_Dest.biHeight);
	param.time_incr = 15;
	// DEC_MEM_REQS dmr;
	// decore((unsigned long)this, DEC_OPT_MEMORY_REQS, &dmr, 0);
	memset(&param.buffers, 0, sizeof(param.buffers));
#if DECORE_VERSION >= 20020303
	switch(m_Info.fourcc)
	{
	case fccDIV3: param.codec_version = 311; break;
	case fccDIVX: param.codec_version = 412; break;
	default:
	case fccDX50: param.codec_version = 500; break;
	}
	param.build_number = 0;
#endif
	m_ulHandle = (unsigned long) this;
	decore(m_ulHandle, DEC_OPT_INIT, &param, m_cFormatBuf);
	return 0;
    }
    virtual int Stop()
    {
	if (!m_ulHandle)
	    return -1;
#if DECORE_VERSION > 20011009
	// older DivX4 release has been broken and crashed here -
	// so release memory only with newer versions
        // and leave the memory leak with the older version
	decore(m_ulHandle, DEC_OPT_RELEASE, 0, 0);
#else
#warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#warning !USING OLD DIVX4 library - memory will be unreleased!
#warning !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#endif
	PluginSetAttrInt(m_Info, divx4str_saturation, m_iLastSaturation);
	PluginSetAttrInt(m_Info, divx4str_contrast, m_iLastContrast);
	PluginSetAttrInt(m_Info, divx4str_brightness, m_iLastBrightness);
	PluginSetAttrInt(m_Info, divx4str_maxauto, m_iMaxAuto);
	PluginSetAttrInt(m_Info, divx4str_postprocessing, m_iLastPPMode);
        m_ulHandle = 0;
        return 0;
    }
    virtual int DecodeFrame(CImage* pImage, const void* src, uint_t size,
			    int is_keyframe, bool render = true,
			    CImage** pOut = 0)
    {
	DEC_FRAME param;

	if (!size || !m_ulHandle)
	    return 0;
	if (m_Dest.biHeight != pImage->GetFmt()->biHeight)
	{
            m_Dest = pImage->GetFmt();
	    Restart();
	}

	// leave here for some time - until all users will use new headers
	param.bitstream = (void*) src;
	param.length = size;

	if (pImage)
	{
	    param.bmp = pImage->Data();
	    param.stride = pImage->Width();
	    param.render_flag = 1;
	}
	else
	{
	    param.bmp = 0;
            param.stride = 0;
	    param.render_flag = 0;
	}

	if (m_bSetFlg)
	{
            m_bSetFlg = false;
	    DEC_SET set;
	    set.postproc_level = m_iLastPPMode * 10;
	    decore(m_ulHandle, DEC_OPT_SETPP, &set, 0);
#ifdef DEC_OPT_GAMMA
	    decore(m_ulHandle, DEC_OPT_GAMMA, (void*)DEC_GAMMA_BRIGHTNESS, (void*)m_iLastBrightness);
	    decore(m_ulHandle, DEC_OPT_GAMMA, (void*)DEC_GAMMA_CONTRAST, (void*)m_iLastContrast);
	    decore(m_ulHandle, DEC_OPT_GAMMA, (void*)DEC_GAMMA_SATURATION, (void*)m_iLastSaturation);
#endif
	}

#ifdef USE_311_DECODER
#if DECORE_VERSION < 20020303
	if(m_bCompat311)
	{
	    //		if(m_bCompatWMV2)
	    //		    decore(m_ulHandle, DEC_OPT_FRAME_WMV2, &param, 0);
	    //		else
	    decore(m_ulHandle, DEC_OPT_FRAME_311, &param, 0);
	}
	else
#endif
#endif
	{
	    decore(m_ulHandle, DEC_OPT_FRAME, &param, 0);
	}
	pImage->SetQuality((float) m_iLastPPMode / DIVX4_MAX_QUALITY);

	return size;
    }
    virtual int SetDestFmt(int bits = 24, fourcc_t csp = 0)
    {
	if (!bits)
	    bits = csp;

	switch(bits)
	{
	case 15:
	case 16:
	case 24:
	case 32:
	    m_Dest.SetBits(bits);
            break;
	case IMG_FMT_I420:
	case IMG_FMT_YV12:
	case IMG_FMT_YUY2:
	case IMG_FMT_I422:
	case IMG_FMT_UYVY:
	    m_Dest.SetSpace(csp);
	    break;
	default:
            return -1;
	}
	if (m_ulHandle)
	    Restart();

	return 0;
    }
    virtual const avm::vector<AttributeInfo>& GetAttrs() const
    {
	switch (m_Dest.biCompression)
	{
	case IMG_FMT_I420:
	case IMG_FMT_YV12:
            // only limited set of supported attrs
	    return m_YvAttrs;
	}
        return m_Info.decoder_info;
    }
    virtual int GetValue(const char* name, int* value) const
    {
	if (strcmp(name, divx4str_postprocessing) == 0)
	    *value = m_iLastPPMode;
	else if (strcmp(name, divx4str_maxauto) == 0)
	    *value = m_iMaxAuto;
	else if (strcmp(name, divx4str_brightness) == 0)
	    *value = m_iLastBrightness;
	else if (strcmp(name, divx4str_contrast) == 0)
	    *value = m_iLastContrast;
	else if (strcmp(name, divx4str_saturation) == 0)
	    *value = m_iLastSaturation;
	else
	    return -1;

	return 0;
    }
    virtual int SetValue(const char* name, int value)
    {
	if (strcmp(name, divx4str_postprocessing) == 0)
	{
	    m_iLastPPMode = value;
            m_bSetFlg = true;
	    return 0;
	}
	else if (strcmp(name, divx4str_maxauto) == 0 && value >= 0 && value <= 6)
	{
	    m_iMaxAuto = value;
            return 0;
	}
	else if (strcmp(name, divx4str_brightness) == 0 && value >= -128 && value <= 127)
	{
	    m_iLastBrightness = value;
            m_bSetFlg = true;
	    return 0;
	}
	else if (strcmp(name, divx4str_contrast) == 0 && value >= -128 && value <= 127)
	{
	    m_iLastContrast = value;
            m_bSetFlg = true;
	    return 0;
	}
	else if (strcmp(name, divx4str_saturation) == 0 && value >= -128 && value <= 127)
	{
	    m_iLastSaturation = value;
            m_bSetFlg = true;
	    return 0;
	}
	return -1;
    }
};
#endif

#ifdef HAVE_LIBDIVXENCORE
class DIVX_VideoEncoder: public IVideoEncoder
{
    BitmapInfo m_bh;
    BITMAPINFOHEADER m_obh;
    void* m_pHandle;
    int m_iState;
    int m_iQuant;
    int m_iCSP;
    bool m_bRtMode;
public:
    DIVX_VideoEncoder(const CodecInfo& info, int compressor, const BITMAPINFOHEADER& bh)
	:IVideoEncoder(info), m_bh(bh), m_obh(bh), m_iState(0), m_bRtMode(false)
    {
	AVM_WRITE(strDebugEnc, "%s linux encoder\n",
		  (info.fourcc == fccDIVX) ? "DivX4" : "DivX5" );
	m_obh.biCompression = info.fourcc;
	m_obh.biHeight = labs(m_obh.biHeight);
	switch (m_bh.biCompression)
	{
        // supports only bottom-up RGB for encoding
	case 0: m_iCSP = ENC_CSP_RGB24; m_bh.SetDirection(true); break;
	case fccYUY2: m_iCSP = ENC_CSP_YUY2; break;
	case fccYV12: m_iCSP = ENC_CSP_YV12; break;
	case fccI420: m_iCSP = ENC_CSP_I420; break;
	}
    }
    virtual ~DIVX_VideoEncoder() {}
    virtual int SetQuality(int quality)
    {
	AVM_WRITE("DivX4 plugin", "quality: %d\n", quality);
	PluginSetAttrInt(m_Info, "quality2", quality);
	return 0;
    }
    virtual int GetQuality() const
    {
        int q = 0;
	PluginGetAttrInt(m_Info, "quality2", &q);
        return q;
    }
    virtual int Start()
    {
	ENC_PARAM param;
	memset(&param, 0, sizeof(param));
	param.x_dim=m_bh.biWidth;
	//param.y_dim=m_bh.biHeight;// labs(m_bh.biHeight);
	param.y_dim=labs(m_bh.biHeight);
	param.framerate=30;//frames/sec

	PluginGetAttrInt(m_Info, divx4str_bitrate, &param.bitrate);//bits/sec
	PluginGetAttrInt(m_Info, divx4str_rc_period, &param.rc_period);
	PluginGetAttrInt(m_Info, divx4str_rc_reaction_period, &param.rc_reaction_period);
	PluginGetAttrInt(m_Info, divx4str_rc_reaction_ratio, &param.rc_reaction_ratio);
	PluginGetAttrInt(m_Info, divx4str_max_quantizer, &param.max_quantizer);//just guess
	PluginGetAttrInt(m_Info, divx4str_min_quantizer, &param.min_quantizer);
	PluginGetAttrInt(m_Info, divx4str_max_key_interval, &param.max_key_interval);
	PluginGetAttrInt(m_Info, divx4str_quality, &param.quality);

        int deinterlace = 0;
	int bidirect = 0;
	int obmc = 0;
#ifdef IF_SUPPORT_PRO
	// fast deinterlace
	PluginGetAttrInt(m_Info, divx4str_deinterlace, &deinterlace);
        // use bidirectional coding
	PluginGetAttrInt(m_Info, divx4str_bidirect, &bidirect);
        // flag to enable overlapped block motion compensation mode
	PluginGetAttrInt(m_Info, divx4str_obmc, &obmc);
#endif
        param.deinterlace = deinterlace;
#ifndef ENCORE_MAJOR_VERSION
	param.use_bidirect = bidirect;
#if DECORE_VERSION > 2001009
	param.obmc = obmc;
#endif
#else
	memset(&param.extensions, 0, sizeof(param.extensions));

#ifdef IF_SUPPORT_PRO
	param.extensions.use_bidirect = bidirect;
	param.extensions.obmc = obmc;

#if ENCORE_MAJOR_VERSION >= 5010

	PluginGetAttrInt(m_Info, divx4str_enable_crop, &param.extensions.enable_crop);
	PluginGetAttrInt(m_Info, divx4str_crop_left, &param.extensions.crop_left);
	PluginGetAttrInt(m_Info, divx4str_crop_right, &param.extensions.crop_right);
	PluginGetAttrInt(m_Info, divx4str_crop_top, &param.extensions.crop_top);
	PluginGetAttrInt(m_Info, divx4str_crop_bottom, &param.extensions.crop_bottom);

	PluginGetAttrInt(m_Info, divx4str_enable_resize, &param.extensions.enable_resize);
	PluginGetAttrInt(m_Info, divx4str_resize_width, &param.extensions.resize_width);
	PluginGetAttrInt(m_Info, divx4str_resize_height, &param.extensions.resize_height);
	PluginGetAttrInt(m_Info, divx4str_resize_mode, &param.extensions.resize_mode);
        int tmp;
	PluginGetAttrInt(m_Info, divx4str_resize_bicubic_B, &tmp);
        param.extensions.bicubic_B = tmp / (double) 1000.0;
	PluginGetAttrInt(m_Info, divx4str_resize_bicubic_C, &tmp);
        param.extensions.bicubic_C = tmp / (double) 1000.0;

	PluginGetAttrInt(m_Info, divx4str_interlace_mode, &param.extensions.interlace_mode);

	PluginGetAttrInt(m_Info, divx4str_temporal_enable, &param.extensions.temporal_enable);
	PluginGetAttrInt(m_Info, divx4str_temporal_level, &tmp);
        param.extensions.temporal_level = tmp / (double) 1000.0;

	PluginGetAttrInt(m_Info, divx4str_spatial_passes, &param.extensions.spatial_passes);
	PluginGetAttrInt(m_Info, divx4str_spatial_level, &tmp);
        param.extensions.spatial_level = tmp / (double) 1000.0;

#endif // ENCORE_MAJOR_VERSION >= 5010

#endif // IF_SUPPORT_PRO

#endif // ENCORE_MAJOR_VERSION
	if (param.quality == 1)
	{
	    m_bRtMode=true;
	    int quality=GetQuality();
	    m_iQuant=int(1+.003*(10000-quality));
	    AVM_WRITE(strDebugEnc, "New quant: %d\n", m_iQuant);
	    if(m_iQuant>31)
		m_iQuant=31;
	    if(m_iQuant<1)
		m_iQuant=1;
	}
	else
	    m_bRtMode=false;
	//	param.flip=1;
	encore(0, ENC_OPT_INIT, &param, 0);
	m_pHandle=param.handle;
        return 0;
    }
    virtual int Stop()
    {
	if(!m_iState)return -1;
	encore(m_pHandle, ENC_OPT_RELEASE, 0, 0);
	m_pHandle = 0;
        return 0;
    }
    virtual int GetOutputSize() const
    {
	return m_bh.biWidth * labs(m_bh.biHeight) * 4;
    }
    virtual const BITMAPINFOHEADER& GetOutputFormat() const
    {
	return m_obh;
    }
    virtual int EncodeFrame(const CImage* src, void* dest, int* is_keyframe, uint_t* size, int* lpckid=0)
    {
        CImage* ci = 0;
	//src->GetFmt()->Print();
	if (!src->IsFmt(&m_bh))
	{
            // slow conversion
            ci = new CImage(src, &m_bh);
	    //src->GetFmt()->Print();
	    //AVM_WRITE(strDebugEnc, "enconding format doesn't match!\n");
	    //return -1;
	}
	ENC_FRAME param;
	ENC_RESULT result;
	param.bitstream = dest;
	param.colorspace = m_iCSP;
	param.image = (ci) ? ci->Data() : (void*) src->Data();
	param.mvs = 0;
	param.length = GetOutputSize();
	if (m_bRtMode)
	{
	    param.quant=m_iQuant;
            // let encoder decide if frame is INTER/INTRA
	    param.intra=-1;
	    encore(m_pHandle, ENC_OPT_ENCODE_VBR, &param, &result);
	}
	else
	    encore(m_pHandle, ENC_OPT_ENCODE, &param, &result);
	if (is_keyframe)
	    *is_keyframe = result.is_key_frame ? 16 : 0;
	if (size)
	    *size = param.length;
	if (ci)
            ci->Release();
	return 0;
    }
};
#endif

static IVideoEncoder* divx4_CreateVideoEncoder(const CodecInfo& info, fourcc_t compressor, const BITMAPINFOHEADER& bh)
{
    quiet_encore = 1;
#ifdef HAVE_LIBDIVXENCORE
    if (bh.biCompression == 0 && bh.biBitCount != 24)
    {
	divx4_error_set("Unsupported input bit depth");
        return 0;
    }
    switch (bh.biCompression)
    {
    case 0:
    case fccYUY2:
    case fccYV12:
    case fccI420:
    case fccDIVX:
    case fccDX50:
	return new DIVX_VideoEncoder(info, compressor, bh);
    }
#endif
    divx4_error_set("Unsupported input format");
    return 0;
}

static IVideoDecoder* divx4_CreateVideoDecoder(const CodecInfo& info, const BITMAPINFOHEADER& bh, int flip)
{
    quiet_encore = 1;
    if (bh.biSize < 40)
    {
	divx4_error_set("to short biSize");
        return 0;
    }
#ifdef HAVE_LIBDIVXDECORE
    return new DIVX_VideoDecoder(info, bh, flip);
#else
    return 0;
#endif
}

AVM_END_NAMESPACE;

avm::codec_plugin_t avm_codec_plugin_divx4 =
{
    PLUGIN_API_VERSION,

    0,
    avm::PluginGetAttrInt,
    avm::PluginSetAttrInt,
    0,
    0,
    avm::divx4_FillPlugins,
    0,
    0,
    avm::divx4_CreateVideoDecoder,
    avm::divx4_CreateVideoEncoder,
};
