#include "FFVideoEncoder.h"
#include "fillplugins.h"
#include <string.h>
#include <stdio.h>

AVM_BEGIN_NAMESPACE;

FFVideoEncoder::FFVideoEncoder(AVCodec* av, const CodecInfo& info, fourcc_t compressor, const BITMAPINFOHEADER& bh)
    :IVideoEncoder(info), m_pAvCodec(av), m_Caps(CAP_YV12), m_bh(bh), m_obh(bh), m_bOpened(false)
{
    m_obh.biCompression = info.fourcc;
    m_obh.biHeight = labs(m_obh.biHeight);
}

FFVideoEncoder::~FFVideoEncoder()
{
    Stop();
}

int FFVideoEncoder::EncodeFrame(const CImage* src, void* dest, int* is_keyframe, uint_t* size, int* lpckid)
{
    int hr = 0;

    if (!m_bOpened)
    {
	memset(&m_AvContext, 0, sizeof(AVCodecContext));
        m_AvContext.width = m_bh.biWidth;
	m_AvContext.height = m_obh.biHeight;
	m_AvContext.pix_fmt = PIX_FMT_YUV422;
        printf("CODEC opening\n");
	//m_AvContext.pix_fmt = PIX_FMT_RGB24;
	if (avcodec_open(&m_AvContext, m_pAvCodec) < 0)
            return -1;
        else
	    m_bOpened = true;
    }

    CImage* ci = 0;
    printf("CODEC format  0x%x\n", src->Format());
    if (src->Format() != IMG_FMT_YV12)
    {
        printf("Converted\n");
        ci = new CImage(src, IMG_FMT_YV12);
    }
    else
        ci = (CImage*) src;

    AVPicture pic; // YV12 only
    pic.data[0] = ci->Data(0);
    pic.data[1] = ci->Data(2);
    pic.data[2] = ci->Data(1);
    pic.linesize[0] = src->Stride(0);
    pic.linesize[1] = src->Stride(2);
    pic.linesize[2] = src->Stride(1);
    printf("ECDING FF  %p %p %p\n", pic.data[0], pic.data[1], pic.data[2]);

    int rsize = 0;//avcodec_encode_video(&m_AvContext, (unsigned char*) dest, GetOutputSize(), &pic);
    printf("ECDING FF  size %d\n", rsize);
    if (size)
	*size = rsize;
    if (ci != src)
        ci->Release();
    return hr;
}

int FFVideoEncoder::Start()
{
    return 0;
}

int FFVideoEncoder::Stop()
{
    if (m_bOpened)
    {
	avcodec_close(&m_AvContext);
	m_bOpened = false;
    }
    return 0;
}

int FFVideoEncoder::SetQuality(int quality)
{
    if (quality < 0 || quality >10000)
	return - 1;
    m_iQual = quality;
    return 0;
}

int FFVideoEncoder::SetKeyFrame(int freq)
{
    return 0;
}

AVM_END_NAMESPACE;
