#ifndef FFMPEG_FILLPLUGINS_H
#define FFMPEG_FILLPLUGINS_H

#include "infotypes.h"
#include "avm_fourcc.h"

//#include "avcodec.h"
#include "avformat.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h> // remove

AVM_BEGIN_NAMESPACE;

#define FFCSTR(name) \
    static const char* ffstr_ ## name = #name

FFCSTR(bitrate);
FFCSTR(dr1);
FFCSTR(bug_old_msmpeg4);
FFCSTR(bug_xvid_ilace);
FFCSTR(bug_ump4);
FFCSTR(bug_no_padding);
FFCSTR(bug_ac_vlc);
FFCSTR(bug_qpel_chroma);

static void libffmpeg_fill_attr(avm::vector<AttributeInfo>& a, bool fix = true)
{
    a.push_back(AttributeInfo(ffstr_dr1, "Direct Rendering 1", AttributeInfo::Integer, 0, 1, 1));
    if (fix)
    {
	a.push_back(AttributeInfo(ffstr_bug_old_msmpeg4, "Fix old msmpeg4", AttributeInfo::Integer, 0, 1, 0));
	a.push_back(AttributeInfo(ffstr_bug_xvid_ilace, "Fix xvid interlace", AttributeInfo::Integer, 0, 1, 0));
	a.push_back(AttributeInfo(ffstr_bug_ump4, "Fix ump4", AttributeInfo::Integer, 0, 1, 0));
	a.push_back(AttributeInfo(ffstr_bug_no_padding, "Fix no padding", AttributeInfo::Integer, 0, 1, 0));
	a.push_back(AttributeInfo(ffstr_bug_ac_vlc, "Fix ac vlc", AttributeInfo::Integer, 0, 1, 0));
	a.push_back(AttributeInfo(ffstr_bug_qpel_chroma, "Fix qpel chroma", AttributeInfo::Integer, 0, 1, 0));
    }
}

static void libffmpeg_fill_encattr(avm::vector<AttributeInfo>& a)
{
#if 0
    avc_config_t* config;
    avc_config_t* cnf;
    const char* arr[100];
    int i = 0;
    avcodec_getopt(0, 0, &config);
    cnf = config;
    while (cnf->name) {
	char* tmp, *semi, *dealloc;
	switch (cnf->type & FF_CONF_TYPE_MASK)
	{
	case FF_CONF_TYPE_INT:
	    a.push_back(AttributeInfo(cnf->name, cnf->help, AttributeInfo::Integer, (int)cnf->min, (int)cnf->max, (int)cnf->defval));
            break;
	case FF_CONF_TYPE_BOOL:
	    a.push_back(AttributeInfo(cnf->name, cnf->help, AttributeInfo::Integer, 0, 1, (int)cnf->defval));
            break;
	case FF_CONF_TYPE_STRING:
	    i = 0;
            dealloc = 0;
	    if (cnf->defstr)
	    {
		tmp = dealloc = strdup(cnf->defstr);
		if (tmp)
		{
		    while (tmp && (semi = strchr(tmp, ';')))
		    {
			*semi++ = 0;
			arr[i++] = tmp;
			if (i > 97)
			    break; // weird - too many string options ???
			tmp = semi;
		    }
		    arr[i++] = tmp;
		}
		arr[i] = 0;
            }
	    //printf("%d   %s\n", i, cnf->name);
	    a.push_back((i > 1) ? AttributeInfo(cnf->name, cnf->help, arr, (int)cnf->defval)
			: AttributeInfo(cnf->name, cnf->help, AttributeInfo::String));
	    if (dealloc)
		free(dealloc);
            break;
	default:
	    //printf("Unsupported %s\n", cnf->name);
            break;
	}

        cnf++;
    }

    free(config);
#endif
}

static void libffmpeg_add_divx(avm::vector<CodecInfo>& ci)
{
    static const char divx_about[] =
	"FFMPEG LGPL version of popular M$ MPEG-4 video codec v3. "
	"Advanced compression technologies allow it to "
	"compress 640x480x25 video with a perfect "
	"quality into 100-150 kbytes/s ( 3-4 times "
	"less than MPEG-2 ).";

    static const fourcc_t div3_codecs[] = {
	fccDIV3, fccdiv3, fccDIV4, fccdiv4,
	fccDIV5, fccdiv5, fccDIV6, fccdiv6,
	fccMP41, fccMP43, RIFFINFO_MPG3, RIFFINFO_mpg3,
	fccAP41, fccap41, fccAP42, fccap42,
	mmioFOURCC('C', 'O', 'L', '1'), // Cool codec
	mmioFOURCC('c', 'o', 'l', '1'),
	mmioFOURCC('C', 'O', 'L', '0'),
	mmioFOURCC('c', 'o', 'l', '0'),
	mmioFOURCC('3', 'I', 'V', 'D'), // 3ivx.com
	mmioFOURCC('3', 'i', 'v', 'd'),
	0 };

    avm::vector<AttributeInfo> da;
    libffmpeg_fill_attr(da);
    //da.push_back(AttributeInfo("Hue", "Hue", AttributeInfo::Integer, 0, 100));
    //da.push_back(AttributeInfo("Brightness", "Brightness", AttributeInfo::Integer, 0, 100));
    //da.push_back(AttributeInfo("Contrast", "Contrast", AttributeInfo::Integer, 0, 100));

    // for encoding
    ci.push_back(CodecInfo(div3_codecs, "FFMPEG DivX ;-)", "msmpeg4",
			   divx_about, CodecInfo::Plugin, "ffdivx",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));

    static const fourcc_t opendix_codecs[] = {
	fccDIVX, fccdivx, RIFFINFO_XVID, RIFFINFO_XviD, RIFFINFO_xvid,
	fccDIV1, fccdiv1, fccMP4S, fccmp4s, mmioFOURCC('m', 'p', '4', 'v'),
	mmioFOURCC('U', 'M', 'P', '4'),
        mmioFOURCC('3', 'I', 'V', '1'),
	mmioFOURCC('3', 'I', 'V', '2'),
        mmioFOURCC('3', 'i', 'v', '2'),	0x4, 0 };
    ci.push_back(CodecInfo(opendix_codecs, "FFMPEG OpenDivX", "mpeg4",
			   "FFMPEG OpenDivX MPEG-4 codec",
			   CodecInfo::Plugin, "ffodivx",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));

    static const fourcc_t codecs[] = { fccDX50, fccdx50, 0 };
    avm::vector<AttributeInfo> vs;
    libffmpeg_fill_encattr(vs);

    ci.push_back(CodecInfo(codecs, "FFMPEG DivX5", "mpeg4",
			   "FFMPEG DivX 5.0 codec",
			   CodecInfo::Plugin, "ffdx50",
			   CodecInfo::Video,
			   //CodecInfo::Decode,
			   CodecInfo::Both, // working on
			   0, vs, da));

    static const fourcc_t wmv_codecs[] = { fccWMV1, fccwmv1, 0 };
    ci.push_back(CodecInfo(wmv_codecs, "FFMPEG Windows Media Video 7", "wmv1",
			   "FFMPEG Windows Media Video 7 codec",
			   CodecInfo::Plugin, "ffwmv1",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));

    static const fourcc_t wmv2_codecs[] = { fccWMV2, fccwmv2, 0 };
    ci.push_back(CodecInfo(wmv2_codecs, "FFMPEG Windows Media Video 8", "wmv2",
			   "FFMPEG Windows Media Video 7 codec",
			   CodecInfo::Plugin, "ffwmv2",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));

    static const fourcc_t mp_codecs[] = { fccMP42, fccmp42, fccDIV2, fccdiv2, 0 };
    ci.push_back(CodecInfo(mp_codecs, "FFMPEG M$ MPEG-4 v2", "msmpeg4v2",
			   "FFMPEG M$ MPEG-4 v2 codec",
			   CodecInfo::Plugin, "ffmp42",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));

    static const fourcc_t mpeg12_codecs[] = {
	RIFFINFO_MPG1, RIFFINFO_MPG2,
	0x10000001, 0x10000002, // mplayer's MPEG-PES 1/2
	0
    };
    ci.push_back(CodecInfo(mpeg12_codecs, "FFMPEG MPEG 1/2", "mpegvideo",
			   "FFMPEG MPEG1/2 decoder",
			   CodecInfo::Plugin, "ffmpeg",
			   CodecInfo::Video, CodecInfo::Decode,  0, 0, da));
}

static void libffmpeg_add_vdec(avm::vector<CodecInfo>& ci)
{
    avm::vector<AttributeInfo> da;
    libffmpeg_fill_attr(da, false);

    static const fourcc_t mjpg_codecs[] = {
	fccMJPG, fccmjpg,
	mmioFOURCC('A', 'V', 'R', 'n'),
	mmioFOURCC('A', 'V', 'D', 'J'),
	mmioFOURCC('J', 'P', 'E', 'G'),
	mmioFOURCC('j', 'p', 'e', 'g'),
	mmioFOURCC('m', 'j', 'p', 'b'),
	0 };
    ci.push_back(CodecInfo(mjpg_codecs, "FFMPEG Motion JPEG", "mjpeg",
			   "FFMPEG Motion JPEG",
			   CodecInfo::Plugin, "ffmjpeg",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));

    static const fourcc_t u263_codecs[] = { fccU263, fccu263, fcch263, fccH263, 0 };
    ci.push_back(CodecInfo(u263_codecs, "FFMPEG H263+", "h263",
			   "FFMPEG H263+ codec",
			   CodecInfo::Plugin, "ffh263",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));

    static const fourcc_t i263_codecs[] = { fccI263, fcci263, 0 };
    ci.push_back(CodecInfo(i263_codecs, "FFMPEG I263", "h263i",
			   "FFMPEG I263 codec",
			   CodecInfo::Plugin, "ffi263",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));


    static const fourcc_t pim_codecs[] = { fccPIM1, 0 };
    ci.push_back(CodecInfo(pim_codecs, "FFMPEG PinnacleS PIM1", "mpegvideo",
			   "FFMPEG PinnacleS PIM1",
			   CodecInfo::Plugin, "ffpim1",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));


    static const fourcc_t dvsd_codecs[] = {
	fccDVSD, fccdvsd, fccdvhd, fccdvsl,
	mmioFOURCC('D', 'V', 'C', 'S'),
	mmioFOURCC('d', 'v', 'c', 's'),
	mmioFOURCC('d', 'v', 'c', 'p'),
	mmioFOURCC('d', 'v', 'c', ' '),
	0
    };
    ci.push_back(CodecInfo(dvsd_codecs, "FFMPEG DV Video", "dvvideo",
			   "FFMPEG DV Video decoder", CodecInfo::Plugin, "ffdv",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));
#if 1
    // wait until it gets fixed
    static const fourcc_t huf_codecs[] = { fccHFYU, 0 };
    ci.push_back(CodecInfo(huf_codecs, "FFMPEG Huffyuv", "huffyuv",
			   "FFMPEG Huffyuv codec",
			   CodecInfo::Plugin, "ffhufyuv",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));
#endif
    static const fourcc_t svq1_codecs[] = { mmioFOURCC('S', 'V', 'Q', '1'), 0 };
    ci.push_back(CodecInfo(svq1_codecs, "FFMPEG SVQ1", "svq1",
			   "FFMPEG Sorenson1 decoder", CodecInfo::Plugin, "ffsvq1",
			   CodecInfo::Video, CodecInfo::Decode, 0, 0, da));
}

static void libffmpeg_add_ac3dec(avm::vector<CodecInfo>& ci)
{
    static const fourcc_t ac3_codecs[] = { 0x2000, 0 };
    ci.push_back(CodecInfo(ac3_codecs, "FFMPEG AC3", "ac3",
			   "AC3 audio codec", CodecInfo::Plugin, "ffac3",
			   CodecInfo::Audio, CodecInfo::Decode));
}

static void libffmpeg_add_adec(avm::vector<CodecInfo>& ci)
{
    static const fourcc_t mpg_codecs[] = { 0x55, 0x50,
    mmioFOURCC('.', 'm', 'p', '3'),
    0 };
    ci.push_back(CodecInfo(mpg_codecs, "FFMPEG MPEG Layer-3", "mp2",
			   "FFMPEG MPEG Layer-III audio decoder",
			   CodecInfo::Plugin, "ffmp3",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t alaw_codecs[] = { 0x06, 0 };
    ci.push_back(CodecInfo(alaw_codecs, "FFMPEG ALaw", "pcm_alaw",
			   "FFMPEG ALaw audio decoder",
			   CodecInfo::Plugin, "ffalaw",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t ulaw_codecs[] = { 0x07, 0 };
    ci.push_back(CodecInfo(ulaw_codecs, "FFMPEG uLaw", "pcm_mulaw",
			   "FFMPEG uLaw audio decoder",
			   CodecInfo::Plugin, "ffmulaw",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t wmav1_codecs[] = { 0x160, 0 };
    ci.push_back(CodecInfo(wmav1_codecs, "FFMPEG WMA v1", "wmav1",
			   "FFMPEG Window Media Audio v1 decoder",
			   CodecInfo::Plugin, "ffwmav1",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t wmav2_codecs[] = { 0x161, 0 };
    ci.push_back(CodecInfo(wmav2_codecs, "FFMPEG WMA v2", "wmav2",
			   "FFMPEG Window Media Audio v2 decoder",
			   CodecInfo::Plugin, "ffwmav2",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t ms_codecs[] = { 0x02, 0 };
    ci.push_back(CodecInfo(ms_codecs, "FFMPEG MS ADPCM", "adpcm_ms",
			   "FFMPEG MS ADPCM audio decoder",
			   CodecInfo::Plugin, "ffadpcmms",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t imaqt_codecs[] = { 0x11, 0x200, 0 }; // unsure
    ci.push_back(CodecInfo(imaqt_codecs, "FFMPEG IMA Qt", "adpcm_ima_qt",
			   "FFMPEG IMA Qt audio decoder",
			   CodecInfo::Plugin, "ffimawav",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t imawav_codecs[] = { 0x11, 0x200, 0 }; // unsure
    ci.push_back(CodecInfo(imawav_codecs, "FFMPEG IMA WAV", "adpcm_ima_wav",
			   "FFMPEG IMA WAV audio decoder",
			   CodecInfo::Plugin, "ffimaqt",
			   CodecInfo::Audio, CodecInfo::Decode));

    static const fourcc_t dvaudio_codecs[] = { ('D' << 8) | 'A', 0 };
    ci.push_back(CodecInfo(dvaudio_codecs, "FFMPEG DV Audio", "dvaudio",
			   "FFMPEG DV Audio decoder",
			   CodecInfo::Plugin, "ffdva",
			   CodecInfo::Audio, CodecInfo::Decode));

}

static void ffmpeg_FillPlugins(avm::vector<CodecInfo>& ci)
{
    // video
    libffmpeg_add_divx(ci);
    libffmpeg_add_vdec(ci);

    // audio
#ifdef HAVE_FFMPEG_A52
    libffmpeg_add_ac3dec(ci);
#endif
    libffmpeg_add_adec(ci);
}

AVM_END_NAMESPACE;

#endif // FFMPEG_FILLPLUGINS_H
