#include "picprop.h"
#include "v4lxif.h"
#include <stdio.h>
#include <qlabel.h>
#include <qslider.h>

#define DECLARE_REGISTRY_SHORTCUT
#include "configfile.h"

/*
 *  Constructs a PicturePropDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
PicturePropDialog::PicturePropDialog(v4lxif* pDev)
    : PicPropDialog( 0, 0, FALSE, 0), m_pDev(pDev)
{
    connect(m_pSBrightness, SIGNAL(valueChanged(int)), this, SLOT(changeBrightness(int)));
    connect(m_pSConstrast, SIGNAL(valueChanged(int)), this, SLOT(changeContrast(int)));
    connect(m_pSHue, SIGNAL(valueChanged(int)), this, SLOT(changeHue(int)));
    connect(m_pSSaturation, SIGNAL(valueChanged(int)), this, SLOT(changeSaturation(int)));

    m_pSBrightness->setValue(RI("Image\\Brightness", 0));
    m_pSConstrast->setValue((RI("Image\\Contrast", 255)-255)/2);
    m_pSSaturation->setValue((RI("Image\\Color", 255)-255)/2);
    m_pSHue->setValue(RI("Image\\Hue", 0));    
//    printf("Reading: %d %d %d %d\n",
//	RI("Image\\Brighness", 0),
//	RI("Image\\Contrast", 255),
//	RI("Image\\Color", 255),
//	RI("Image\\Hue", 0)
//	);
    m_pDev->setPicBrightness(RI("Image\\Brightness", 0));
    m_pDev->setPicConstrast(RI("Image\\Contrast", 255));
    m_pDev->setPicColor(RI("Image\\Color", 255));
    m_pDev->setPicHue(RI("Image\\Hue", 0));
}

void PicturePropDialog::changeBrightness(int val)
{
    char s[128];
    sprintf(s, "Brightness: %d", val);
    m_pLBrightness->setText(s);
    m_pDev->setPicBrightness(val);
}
void PicturePropDialog::changeContrast(int val)
{
    char s[128];
    sprintf(s, "Contrast: %d", val);
    m_pLContrast->setText(s);
    val=2*(val+128);
    m_pDev->setPicConstrast(val);
}
void PicturePropDialog::changeHue(int val)
{
    char s[128];
    sprintf(s, "Hue: %d", val);
    m_pLHue->setText(s);
    m_pDev->setPicHue(val);
}
void PicturePropDialog::changeSaturation(int val)
{
    char s[128];
    sprintf(s, "Saturation: %d", val);
    m_pLSaturation->setText(s);
    val=2*(val+128);
    m_pDev->setPicColor(val);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
PicturePropDialog::~PicturePropDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

void PicturePropDialog::accept()
{
    WI("Image\\Brightness", m_pSBrightness->value());
    WI("Image\\Color", 256+2*m_pSSaturation->value());
    WI("Image\\Contrast", 256+2*m_pSConstrast->value());
    WI("Image\\Hue", m_pSHue->value());
//    printf("Writing: %d %d %d %d\n",
//	m_pSBrightness->value(),
//	256+2*m_pSConstrast->value(),
//	256+2*m_pSSaturation->value(),
//	m_pSHue->value()
//	);	
    return QDialog::accept();
}

#include "picprop.moc"
