#include "vidconf.h"
#include "v4lwindow.h"
#include "capproc.h"
#include "codecdialog.h"

#include <avm_fourcc.h>
#include <utils.h>
#include <cpuinfo.h>
#include <creators.h>
#define DECLARE_REGISTRY_SHORTCUT
#include <configfile.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>

#include <stdio.h>
#include <stdlib.h> // atof

#include "v4lxif.h"

/*
 *  Constructs a VidConfig which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
VidConfig::VidConfig( QWidget* parent, v4lxif* v4l, V4LWindow* w)
    : VidcapConfigDialog( parent, "Configure QtVidcap", true, 0 ), m_pV4lWin(w)
{
    char str[256];//any integers longer than this out there?

    _CapDevice->setText(RS("CaptureDevice", "/dev/video"));
    _AudDevice->setText(RS("AudioDevice", "/dev/dsp"));
    //ask hw
    //fixme
    int i = 0;
    while (cv_dimensions[i].id >= 0)
	_Resolution->insertItem(cv_dimensions[i++].lname);

    _Resolution->setCurrentItem( RI("Resolution", 0) );

    _VideoColorMode->insertItem( tr( "PAL" ) );
    _VideoColorMode->insertItem( tr( "NTSC" ) );
    _VideoColorMode->insertItem( tr( "SECAM" ) );
    _VideoColorMode->insertItem( tr( "Auto" ) );
    _VideoColorMode->setCurrentItem( RI("ColorMode", 0) );

    _VideoChannel->clear();
    if (v4l)
        for(int j=0; j<v4l->capCapChannelC(); j++)
	    _VideoChannel->insertItem(v4l->capChannelName(j));
    _VideoChannel->setCurrentItem( RI("Channel", 1) );

    _Colorspace->insertItem( tr( "YV12" ) );
    _Colorspace->insertItem( tr( "YUY2" ) );
    _Colorspace->insertItem( tr( "RGB15LE" ) );
    _Colorspace->insertItem( tr( "RGB24" ) );
    _Colorspace->insertItem( tr( "RGB32" ) );
    _Colorspace->setCurrentItem( RI("Colorspace", 0) );

    _FileName->setText( RS("FileName", "./movie.avi") );

    sprintf(str, "%d", RI("SegmentSize", 1000*1000));
    _SegmentSize->setText(str);

    _HaveSegmented->setChecked( RI("IsSegmented", 1) );

    _HaveAudio->setChecked( RI("HaveAudio", 1) );

    _listChan->insertItem( tr( "Mono" ) );
    _listChan->insertItem( tr( "Stereo" ) );
    _listChan->insertItem( tr( "Lang1" ) );
    _listChan->insertItem( tr( "Lang2" ) );
    _listChan->setCurrentItem(RI("SndChannels", 0));

    _listFreq->insertItem( tr( "48000 Hz" ) );
    _listFreq->insertItem( tr( "44100 Hz" ) );
    _listFreq->insertItem( tr( "32000 Hz" ) );
    _listFreq->insertItem( tr( "22050 Hz" ) );
    _listFreq->insertItem( tr( "16000 Hz" ) );
    _listFreq->insertItem( tr( "12000 Hz" ) );
    _listFreq->insertItem( tr( "11025 Hz" ) );
    _listFreq->insertItem( tr( " 8000 Hz" ) );
    _listFreq->setCurrentItem(RI("Frequency", 0));

    _listSamp->insertItem( tr( "16 bit" ) );
    _listSamp->insertItem( tr( "8 bit" ) );
    _listSamp->setCurrentItem(RI("SampleSize", 0));

    _info.compressor = RI("Compressor", fccDIV3);
    _info.cname = RS("Codec", "select");
    _info.quality = RI("Quality", 9500);
    _info.keyfreq = RI("Keyframe", 15);

    char ft[4];
    sprintf(str, "%s: %.4s / %d", _info.cname.c_str(),
	    avm_set_le32(ft, _info.compressor), _info.quality/100);
    _codecName->setText(str);

    _chkTime->setChecked(RI("LimitTime", 0));
    _chkFileSize->setChecked(RI("LimitSize", 0));

    sprintf(str, "%d", RI("SizeLimit", 2000000));
    _sizeLimit->setText(str);
    sprintf(str, "%d", RI("TimeLimit", 3600));
    _timeLimit->setText(str);

    _fps->insertItem( tr( "12" ) );
    _fps->insertItem( tr( "15" ) );
    _fps->insertItem( tr( "18" ) );
    _fps->insertItem( tr( "20" ) );
    _fps->insertItem( tr( "23.975" ) );
    _fps->insertItem( tr( "24" ) );
    _fps->insertItem( tr( "25" ) );
    _fps->insertItem( tr( "29.970" ) );
    _fps->insertItem( tr( "30" ) );
    char b[30];
    sprintf(b, "%2.3f", RI("FPS", 25000) / 1000.0);
    _fps->setEditText(b);

}

/*
 *  Destroys the object and frees any allocated resources
 */
VidConfig::~VidConfig()
{
    // no need to delete child widgets, Qt does it all for us
}


void VidConfig::accept()
{
    savePage(0);
    savePage(1);
    savePage(2);
    savePage(3);
    if (m_pV4lWin) m_pV4lWin->setupDevice();
    QDialog::accept();
}

void VidConfig::apply()
{
#if QT_VERSION>=219
    savePage(tabWidget->currentPageIndex());
#else
    savePage(0);
    savePage(1);
    savePage(2);
    savePage(3);
#endif
    if (m_pV4lWin) m_pV4lWin->setupDevice();
}

void VidConfig::savePage(int page)
{
    switch(page)
    {
    case 0:
	WS("CapDev", _CapDevice->text());
	WS("AudioDevice", _AudDevice->text());
	WI("Resolution", _Resolution->currentItem());
	WI("Channel", _VideoChannel->currentItem());
	WI("ColorMode", _VideoColorMode->currentItem());
	WI("Colorspace", _Colorspace->currentItem());
	break;
    case 1:
	WS("FileName", _FileName->text());
	WI("SegmentSize", _SegmentSize->text());
	WI("IsSegmented", _HaveSegmented->isChecked());
	WI("FPS", int(atof(_fps->currentText()) * 1000));
	break;
    case 2:
	WI("Compressor", _info.compressor);
	WS("Codec", _info.cname);
	WI("Quality", _info.quality);
	WI("Keyframe", _info.keyfreq);
	WI("HaveAudio", _HaveAudio->isChecked());
	WI("SampleSize", _listSamp->currentItem());
	WI("Frequency", _listFreq->currentItem());
	WI("SndChannels", _listChan->currentItem());
	break;
    case 3:
	WI("TimeLimit", _timeLimit->text());
	WI("SizeLimit", _sizeLimit->text());
	WI("LimitTime", _chkTime->isChecked());
	WI("LimitSize", _chkFileSize->isChecked());
	break;
    }
}
/*
 * public slot
 */
void VidConfig::change_filename()
{
    QString str = QFileDialog::getOpenFileName( _FileName->text(),
						tr( "*.avi;*.AVI" ),
						0, 0
#if QT_VERSION > 220
						, tr( "Select destination file" )
#endif
					      );
    if (!str.isNull())
	_FileName->setText(str);
}
/*
 * public slot
 */
void VidConfig::change_codec()
{
    int bpp = 0;
    int w = 768;
    int h = 576;

    switch(_Colorspace->currentItem())
    {
    case cspYV12:
	bpp = fccYV12;
	break;
    case cspYUY2:
	bpp = fccYUY2;
	break;
    case cspI420:
	bpp = fccI420;
	break;
    case cspRGB15:
	bpp = 15;
	break;
    case cspRGB24:
	bpp = 24;
	break;
    case cspRGB32:
	bpp = 32;
	break;
    }

    int i = _Resolution->currentItem();
    if (i >= 0 && i < 9)
    {
	w = cv_dimensions[i++].w;
	h = cv_dimensions[i++].h;
    }

    BITMAPINFOHEADER bih;
    bih.biCompression = 0xffffffff;
    // just to fill video_codecs list
    avm::CreateDecoderVideo(bih, 0, 0);

    avm::vector<avm::CodecInfo> codecs;
    avm::BitmapInfo bi(w, h, bpp);
    avm::vector<avm::CodecInfo>::const_iterator it;
    int sel = 0;
    i = 0;
    for (it = video_codecs.begin(); it != video_codecs.end(); it++)
    {
	//printf("CHECKING %s  %x\n", it->GetName(), bpp);
	if (!(it->direction & avm::CodecInfo::Encode))
	    continue;
	avm::IVideoEncoder* enc = avm::CreateEncoderVideo(*it, bi);
        // checking if this code is the one we have asked for...
	if (!enc)
	    continue;
	bool ok = (strcmp(it->GetName(), enc->GetCodecInfo().GetName()) == 0);
	avm::FreeEncoderVideo(enc);
	if (!ok)
            continue;
	codecs.push_back(*it);
	if (it->fourcc == _info.compressor)
	    sel=i;
	i++;
    }

    //printf("SEL %d\n", sel);
    QavmCodecDialog conf(this, codecs);
    conf.setCurrent(sel);

    int gx = RI("CodecGeometryX", conf.x());
    int gy = RI("CodecGeometryY", conf.y());
    int gw = RI("CodecGeometryWidth", conf.width());
    int gh = RI("CodecGeometryHeight", conf.height());
    int gm = RI("CodecGeometryMaximized", 0);

    conf.resize( gw, gh );
    conf.move( gx, gy);
    if (gm)
	conf.showMaximized();

    if(conf.exec()==QDialog::Accepted)
    {
	WI("CodecGeometryX", conf.x());
	WI("CodecGeometryY", conf.y());
	WI("CodecGeometryWidth", conf.width());
	WI("CodecGeometryHeight", conf.height());
#if QT_VERSION > 220
	WI("CodecGeometryMaximized", conf.isMaximized());
#endif
	_info=conf.getInfo();
	char str[256];
	sprintf(str, "%.4s/%d", (char*) &_info.compressor, _info.quality/100);
	_codecName->setText(str);
    }
}
/*
 * public slot
 */
void VidConfig::toggle_audio(bool res)
{
    _listSamp->setEnabled(res);
    _listFreq->setEnabled(res);
    _listChan->setEnabled(res);
}
/*
 * public slot
 */
void VidConfig::toggle_segmented(bool res)
{
    _SegmentSize->setEnabled(res);
}

void VidConfig::toggle_limitsize(bool res)
{
    _sizeLimit->setEnabled(res);
}

void VidConfig::toggle_limittime(bool res)
{
    _timeLimit->setEnabled(res);
}


#include "vidconf.moc"
