#include "vidconf_p.h"

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qvalidator.h>
#include <qwhatsthis.h>

/*
 *  Constructs a VidcapConfigDialog which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
VidcapConfigDialog::VidcapConfigDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QavmOkDialog( parent, name, modal, fl )
{
    setCaption( tr( "Avicap configuration" ) );
#if QT_VERSION > 220
    setSizeGripEnabled( TRUE );
#endif

    tabWidget = new QTabWidget( this );

    QWidget* w = new QWidget( tabWidget );
    QGridLayout* gl = new QGridLayout( w, 1, 1 );
    gl->setSpacing( 5 );
    gl->setMargin( 5 );

    // capture device
    int row = 0;
    QLabel* l = new QLabel( tr( "BTTV capture device:" ), w );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, row, 0 );

    _CapDevice = new QLineEdit( w );
    QWhatsThis::add(  _CapDevice, tr( "Do not change default value ( /dev/video ) unless you exactly know what you're doing." ) );
    gl->addWidget( _CapDevice, row, 1 );

    // audio device
    row++;
    l = new QLabel( tr( "Audio device:" ), w );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, row, 0 );

    _AudDevice = new QLineEdit( w );
    QWhatsThis::add(  _AudDevice, tr( "Do not change default value ( /dev/dsp ) unless you exactly know what you're doing." ) );
    gl->addWidget( _AudDevice, row, 1 );

    // channel
    row++;
    l = new QLabel( tr( "Video channel:" ), w );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, row, 0 );

    _VideoChannel = new QComboBox( FALSE, w );
    gl->addWidget( _VideoChannel, row, 1 );
    tabWidget->insertTab( w, tr( "&Device" ) );

    // color mode
    row++;
    l = new QLabel( tr( "Color mode:" ), w );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, row, 0 );

    _VideoColorMode = new QComboBox( FALSE, w );
    gl->addWidget( _VideoColorMode, row, 1 );

    // resolution
    row++;
    l = new QLabel( tr( "Resolution:" ), w );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, row, 0 );

    _Resolution = new QComboBox( FALSE, w );
    QWhatsThis::add(  _Resolution, tr( "Resolutions higher than 384x288 aren't supported." ) );
    gl->addWidget( _Resolution, row, 1 );

    // color space
    row++;
    l = new QLabel( tr( "Capture color space:" ), w );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, row, 0 );

    _Colorspace = new QComboBox( FALSE, w );
    //QWhatsThis::add(  _Colorspace, tr( "Resolutions higher than 384x288 aren't supported." ) );
    gl->addWidget( _Colorspace, row, 1 );

    w = new QWidget( tabWidget );
    QVBoxLayout* vbl36 = new QVBoxLayout( w );
    vbl36->setMargin( 5 );

    QGroupBox* gb = new QGroupBox( tr( "Destination file" ), w );
    vbl36->addWidget( gb );
    gb->setColumnLayout( 0, Qt::Vertical );
    QHBoxLayout* hbl = new QHBoxLayout( gb->layout() );
    hbl->setSpacing( 5 );
    _ChangeFile = new QPushButton( tr( "C&hange..." ), gb );
    hbl->addWidget( _ChangeFile );
    _FileName = new QLineEdit( gb );
    hbl->addWidget( _FileName );


    gb = new QGroupBox( tr( "Segmentation" ), w );
    vbl36->addWidget( gb );
    gb->setColumnLayout( 0, Qt::Vertical );
    QToolTip::add( gb, tr( "" ) );
    QWhatsThis::add( gb, tr( "Segmentation allows to write more than 2 Gb of data ( limit for an AVI file ) during one session. When it's turned on, program will create a sequence of AVI files, automatically switching them when their sizes become larger than user-specified value." ) );

    gl = new QGridLayout( gb->layout(), 1, 1 );
    gl->setSpacing( 5 );

    l = new QLabel( tr( "Segment size ( KBytes ):" ), gb );
    gl->addWidget( l, 1, 0 );

    _HaveSegmented = new QCheckBox( tr( "&Write segmented file" ), gb );
    gl->addWidget( _HaveSegmented, 0, 0 );

    _SegmentSize = new QLineEdit( gb );
    _SegmentSize->setEnabled( FALSE );
    gl->addWidget( _SegmentSize, 1, 1 );

    tabWidget->insertTab( w, tr( "&File" ) );



    w = new QWidget( tabWidget );
    QVBoxLayout* vbl37 = new QVBoxLayout( w );
    vbl37->setMargin( 5 );

    gb = new QGroupBox( tr( "Video compression format" ), w );
    vbl37->addWidget( gb );
    gb->setColumnLayout( 0, Qt::Vertical );

    hbl = new QHBoxLayout( gb->layout() );
    hbl->setSpacing( 5 );

    l = new QLabel( tr( "Current format:" ), gb );
    hbl->addWidget( l );

    _codecName = new QLabel( gb );
    _codecName->setText( tr( "" ) );
    _codecName->setFrameShadow( QLabel::Sunken );
    _codecName->setFrameShape( QLabel::Panel );
    _codecName->setAlignment( QLabel::AlignCenter );
    hbl->addWidget( _codecName );

    _ChangeCodec = new QPushButton( gb );
    _ChangeCodec->setText( tr( "C&hange..." ) );
    hbl->addWidget( _ChangeCodec );


    gb = new QGroupBox( tr( "Video frame rate" ), w );
    vbl37->addWidget( gb );
    gb->setColumnLayout(0, Qt::Vertical);

    gl = new QGridLayout( gb->layout(), 1, 1 );
    gl->setSpacing( 5 );

    l = new QLabel( tr( "Frames per second:" ), gb );
    gl->addWidget( l, 0, 0 );

    _fps = new QComboBox( gb );
#if QT_VERSION > 220
    _fps->setEditable( true );
#endif
    QValidator* qv = new QDoubleValidator( 1, 30, 3, gb );
    _fps->setValidator( qv );
    _fps->setEditText( "25" );
    gl->addWidget( _fps, 0, 1 );

    gb = new QGroupBox( tr( "Video cropping" ), w );
    vbl37->addWidget( gb );
    gb->setColumnLayout(0, Qt::Vertical);

    tabWidget->insertTab( w, tr( "&Video" ) );


    w = new QWidget( tabWidget );
    vbl37 = new QVBoxLayout( w );
    vbl37->setMargin( 5 );

    gb = new QGroupBox( tr( "Audio compression format" ), w );
    vbl37->addWidget( gb );
    gb->setColumnLayout( 0, Qt::Vertical );

    gb = new QGroupBox( tr( "Audio format" ), w );
    vbl37->addWidget( gb );
    gb->setColumnLayout( 0, Qt::Vertical );

    gl = new QGridLayout( gb->layout(), 1, 1 );
    gl->setSpacing( 5 );

    l = new QLabel( tr( "Frequency:" ), gb );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, 1, 0 );

    l = new QLabel( tr( "Sample size:" ), gb );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, 1, 2 );

    l = new QLabel( tr( "Channels:" ), gb );
    l->setAlignment( QLabel::AlignRight | QLabel::AlignVCenter );
    gl->addWidget( l, 0, 2 );

    _listSamp = new QComboBox( FALSE, gb );
    _listSamp->setEnabled( FALSE );
    gl->addWidget( _listSamp, 1, 3 );

    _HaveAudio = new QCheckBox( tr( "Capture audi&o" ), gb );
    gl->addMultiCellWidget( _HaveAudio, 0, 0, 0, 1 );

    _listFreq = new QComboBox( FALSE, gb );
    _listFreq->setEnabled( FALSE );
    gl->addWidget( _listFreq, 1, 1 );

    _listChan = new QComboBox( FALSE, gb );
    _listChan->setEnabled( FALSE );
    gl->addWidget( _listChan, 0, 3 );

    tabWidget->insertTab( w, tr( "&Audio" ) );



    w = new QWidget( tabWidget );
    QVBoxLayout* vbl38 = new QVBoxLayout( w );
    vbl38->setMargin( 5 );

    gb = new QGroupBox( tr( "Limit capture process by:" ), w );
    vbl38->addWidget( gb );
    gb->setColumnLayout( 0, Qt::Vertical );

    gl = new QGridLayout( gb->layout(), 1, 1 );
    gl->setSpacing( 5 );

    _chkTime = new QCheckBox( tr( "&Time" ), gb );
    gl->addWidget( _chkTime, 0, 0 );

    _timeLimit = new QLineEdit( gb );
    _timeLimit->setEnabled( FALSE );
    gl->addWidget( _timeLimit, 0, 1 );

    l = new QLabel( tr( "seconds" ), gb );
    gl->addWidget( l, 0, 2 );

    _chkFileSize = new QCheckBox( tr( "File &size" ), gb );
    gl->addWidget( _chkFileSize, 1, 0 );

    _sizeLimit = new QLineEdit( gb );
    _sizeLimit->setEnabled( FALSE );
    gl->addWidget( _sizeLimit, 1, 1 );

    l = new QLabel( tr( "KBytes" ), gb );
    gl->addWidget( l, 1, 2 );

    tabWidget->insertTab( w, tr( "&Limits" ) );

    gridLayout()->addWidget( tabWidget, 0, 0 );
    setApplyEnabled( TRUE );

    // signals and slots connections
    connect( _HaveAudio, SIGNAL( toggled(bool) ), this, SLOT( toggle_audio(bool) ) );
    connect( _HaveSegmented, SIGNAL( toggled(bool) ), this, SLOT( toggle_segmented(bool) ) );
    connect( _ChangeFile, SIGNAL( clicked() ), this, SLOT( change_filename() ) );
    connect( _ChangeCodec, SIGNAL( clicked() ), this, SLOT( change_codec() ) );
    connect( _chkTime, SIGNAL( toggled(bool) ), this, SLOT( toggle_limittime(bool) ) );
    connect( _chkFileSize, SIGNAL( toggled(bool) ), this, SLOT( toggle_limitsize(bool) ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
VidcapConfigDialog::~VidcapConfigDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

#include "vidconf_p.moc"
